--
--		TotemCallSetup 1.00 - Steve Kaye
--
--		Allows the setup of the Shaman "Call of" spells for those
--		who have hidden the default Blizzard totem UI.
--

TotemCallSetup = LibStub("AceAddon-3.0"):NewAddon("TotemCallSetup", "AceConsole-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("TotemCallSetup", false)

-- The available options
local options = {
	name = "TotemCallSetup",
	handler = TotemCallSetup,
	icon =  "Interface\\Icons\\Ability_Warrior_Defensivestance.jpg",
	type = "group",	
	args = {
		version = {
			order = 1,
			type = "description",
			name = "X"
		},
		description = {
			order = 2,
			type = "description",
			name = "Descriptive text"
		},
		ancestors = {
			name = L["Call of the Ancestors"],
			type = "group",
			disabled = function () return not TotemCallSetup:IsSpellKnown(L["Call of the Ancestors"]) end,
			args = {
				description = {
					order = 1,
					type = "description",
					name = L["Call of the Ancestors"]
				},
				space = {
					order = 2,
					type = "description",
					name = " "
				},
				earth = {
					order = 3,
					type = "select",
					name = L["Earth Totem"],
					values = function() return TotemCallSetup:GetValidTotems(138) end,
					set = function(self, val) SetMultiCastSpell(138, val) end,
					get = function() return TotemCallSetup:GetSelectedTotem(138) end,
				},
				fire = {
					order = 4,
					type = "select",
					name = L["Fire Totem"],
					values = function() return TotemCallSetup:GetValidTotems(137) end,
					set = function(self, val) SetMultiCastSpell(137, val) end,
					get = function() return TotemCallSetup:GetSelectedTotem(137) end,
				},
				water = {
					order = 5,
					type = "select",
					name = L["Water Totem"],
					values = function() return TotemCallSetup:GetValidTotems(139) end,
					set = function(self, val) SetMultiCastSpell(139, val) end,
					get = function() return TotemCallSetup:GetSelectedTotem(139) end,
				},
				air = {
					order = 6,
					type = "select",
					name = L["Air Totem"],
					values = function() return TotemCallSetup:GetValidTotems(140) end,
					set = function(self, val) SetMultiCastSpell(140, val) end,
					get = function() return TotemCallSetup:GetSelectedTotem(140) end,
				},
			}
		},
		elements = {
			name = L["Call of the Elements"],
			type = "group",
			disabled = function () return not TotemCallSetup:IsSpellKnown(L["Call of the Elements"]) end,
			args = {
				description = {
					order = 1,
					type = "description",
					name = L["Call of the Elements"]
				},
				space = {
					order = 2,
					type = "description",
					name = " "
				},
				earth = {
					order = 3,
					type = "select",
					name = L["Earth Totem"],
					values = function() return TotemCallSetup:GetValidTotems(134) end,
					set = function(self, val) SetMultiCastSpell(134, val) end,
					get = function() return TotemCallSetup:GetSelectedTotem(134) end,
				},
				fire = {
					order = 4,
					type = "select",
					name = L["Fire Totem"],
					values = function() return TotemCallSetup:GetValidTotems(133) end,
					set = function(self, val) SetMultiCastSpell(133, val) end,
					get = function() return TotemCallSetup:GetSelectedTotem(133) end,
				},
				water = {
					order = 5,
					type = "select",
					name = L["Water Totem"],
					values = function() return TotemCallSetup:GetValidTotems(135) end,
					set = function(self, val) SetMultiCastSpell(135, val) end,
					get = function() return TotemCallSetup:GetSelectedTotem(135) end,
				},
				air = {
					order = 6,
					type = "select",
					name = L["Air Totem"],
					values = function() return TotemCallSetup:GetValidTotems(136) end,
					set = function(self, val) SetMultiCastSpell(136, val) end,
					get = function() return TotemCallSetup:GetSelectedTotem(136) end,
				},
			},
		},
		spirits = {
			name = L["Call of the Spirits"],
			type = "group",
			disabled = function () return not TotemCallSetup:IsSpellKnown(L["Call of the Spirits"]) end,
			args = {
				description = {
					order = 1,
					type = "description",
					name = L["Call of the Spirits"]
				},
				space = {
					order = 2,
					type = "description",
					name = " "
				},
				earth = {
					order = 3,
					type = "select",
					name = L["Earth Totem"],
					values = function() return TotemCallSetup:GetValidTotems(142) end,
					set = function(self, val) SetMultiCastSpell(142, val) end,
					get = function() return TotemCallSetup:GetSelectedTotem(142) end,
				},
				fire = {
					order = 4,
					type = "select",
					name = L["Fire Totem"],
					values = function() return TotemCallSetup:GetValidTotems(141) end,
					set = function(self, val) SetMultiCastSpell(141, val) end,
					get = function() return TotemCallSetup:GetSelectedTotem(141) end,
				},
				water = {
					order = 5,
					type = "select",
					name = L["Water Totem"],
					values = function() return TotemCallSetup:GetValidTotems(143) end,
					set = function(self, val) SetMultiCastSpell(143, val) end,
					get = function() return TotemCallSetup:GetSelectedTotem(143) end,
				},
				air = {
					order = 6,
					type = "select",
					name = L["Air Totem"],
					values = function() return TotemCallSetup:GetValidTotems(144) end,
					set = function(self, val) SetMultiCastSpell(144, val) end,
					get = function() return TotemCallSetup:GetSelectedTotem(144) end,
				},
			},
		},
	},
}

-- The command line arguments
local commandLine = {
	name = "TotemCallSetup",
	handler = TotemCallSetup,
	type = "group"
}

function TotemCallSetup:OnInitialize()

	-- Find out if the player is a shaman
	local _, class = UnitClass("player")
	if class == "SHAMAN" then
		-- Get the version number for the TOC
		options.args.version.name = string.format("Version %s", GetAddOnMetadata("TotemCallSetup", "Version"))
		options.args.description.name = GetAddOnMetadata("TotemCallSetup", "Notes")
		
		local acreg = LibStub("AceConfigRegistry-3.0")
		
		-- Register the options
		acreg:RegisterOptionsTable("TotemCallSetup", options)
		acreg:RegisterOptionsTable("TotemCallSetupCommandLine", commandLine)
		
		-- Add the options to the blizzard options interface
		self.OptionsPage = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("TotemCallSetup", "TotemCallSetup")
		
		-- Register the command line
		self:RegisterChatCommand("tcs", "ChatCommand")
	end
end

-- Enable the addon
function TotemCallSetup:OnEnable()
end

function TotemCallSetup:OnDisable()
end

function TotemCallSetup:ChatCommand(input)
	InterfaceOptionsFrame_OpenToCategory(self.OptionsPage)
end

function TotemCallSetup:GetSpellId(spell)
	local spellID = (GetSpellLink(spell) or ""):match("Hspell:(%d+)")
	return spellID and tonumber(spellID)
end

function TotemCallSetup:IsSpellKnown(spellName)

	local spellId = TotemCallSetup:GetSpellId(spellName)
	if spellId ~= nil and IsSpellKnown(spellId) then
		return true
	end
	
	return false
end

function TotemCallSetup:GetValidTotems(slot)

	local validTotems = {}
	local spellName
	
	-- Add an empty value then the user can unselect a totem
	validTotems[0] = " ";
	
	-- Get the list of valid totems
	local t1, t2, t3, t4, t5, t6, t7 = GetMultiCastTotemSpells(slot)
	
	-- Add them to the array
	if t1 ~= nil then
		validTotems[t1] = GetSpellInfo(t1)
	end
	if t2 ~= nil then
		validTotems[t2] = GetSpellInfo(t2)
	end
	if t3 ~= nil then
		validTotems[t3] = GetSpellInfo(t3)
	end
	if t4 ~= nil then
		validTotems[t4] = GetSpellInfo(t4)
	end
	if t5 ~= nil then
		validTotems[t5] = GetSpellInfo(t5)
	end
	if t6 ~= nil then
		validTotems[t6] = GetSpellInfo(t6)
	end
	if t7 ~= nil then
		validTotems[t7] = GetSpellInfo(t7)
	end
	
	return validTotems
end

function TotemCallSetup:GetSelectedTotem(slot)
	local _, globalId, _ = GetActionInfo(slot)
	return globalId
end

